
  THE ANF PROGRAM PARSER
  ======================

  Copyright  2000 Patryk Zadarnowski pat@jantar.org

  Redistribution  and  use  in  source  and  binary  forms,  with  or  without
  modification, are permitted provided  that the following conditions are met:
  (1) Redistributions of  source code must retain the  above copyright notice,
  this list  of conditions and the following  disclaimer.  (2) Redistributions
  in  binary form  must reproduce  the above  copyright notice,  this  list of
  conditions and  the following disclaimer  in the documentation  and/or other
  materials provided  with the distribution.  (3)  The name of  any author may
  not  be used  to  endorse or  promote  products derived  from this  software
  without their specific prior written permission.

  THIS SOFTWARE IS  PROVIDED ``AS IS AND ANY  EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
  AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
  SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
  INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
  CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
  ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

	$Id: ANFParser.lhs,v 1.5 2003/03/31 09:09:38 patrykz Exp $

   --------------------------------------------------------------------------


  OVERVIEW
  ========

  This module implements a simple recursive-descent parser for our concrete
  ANF syntax. The output is annotated with position information, and should be
  stripped before printing. The expression positions are useful for generating
  useful error messages during parsing and typechecking.

> module ANFParser (
>  parse
> ) where

> import Posn
> import Diag
> import ANFSyntax
> import ANFScanner

> parse :: String -> String -> Ep
> parse name content =
>  case parseE (scanTokens name content) of
>   (p, [EOF_TOKEN _]) -> p
>   (_, ERR_TOKEN posn msg:_) -> SYNTAX_ERROR [parseError posn msg]
>   (_, t:_) -> SYNTAX_ERROR [parseError (toknPosn t) msg]
>  where
>   msg = "unexpected tokens after the program entry expression"

	E ::= "if" V "then" E "else" E
	E ::= "letrec" FV "in" E
	E ::= "let" LV
	E ::= V E'

> parseE :: Tokens -> (Ep, Tokens)
> parseE t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseE (IF_TOKEN posn:t1) =
>  let
>   (v,  t2) = parseV     t1
>   (    t3) = parseTHEN  t2
>   (e1, t4) = parseE     t3
>   (    t5) = parseELSE  t4
>   (e2, t6) = parseE     t5
>  in
>   (IFp v e1 e2 posn, t6)
> parseE (LETREC_TOKEN posn:t1) =
>  let
>   (fv, t2) = parseFV t1
>   (    t3) = parseIN t2
>   (e,  t4) = parseE  t3
>  in
>   (LETRECp fv e posn, t4)
> parseE (LET_TOKEN posn:t1) =
>  let
>   (e, t2) = parseLV posn t1
>  in
>   (e, t2)
> parseE t0 =
>  let
>   (v, t1) = parseV    t0
>   (e, t2) = parseE' v t1
>  in
>   (e, t2)

	E' ::= "(" VVopt ")"
	E' ::= "," VV
	E' ::= epsilon

> parseE' :: Vp -> Tokens -> (Ep, Tokens)
> parseE' v (LPAREN_TOKEN posn:t1) =
>  let
>   (vv, t2) = parseVVopt  t1
>   (    t3) = parseRPAREN t2
>  in
>   (RETAp v vv, t3)
> parseE' v (COMMA_TOKEN posn:t1) =
>  let
>   (vv, t2) = parseVV t1
>  in
>   (RETp (v:vv), t2)
> parseE' v t0 = (RETp [v], t0)

	LV ::= XV "=" V LV'

> parseLV :: Posn -> Tokens -> (Ep, Tokens)
> parseLV posn t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseLV posn t0 =
>  let
>   (xv, t1) = parseXV            t0
>   (    t2) = parseASSIGN        t1
>   (v,  t3) = parseV             t2
>   (e,  t4) = parseLV' posn xv v t3
>  in
>   (e, t4)

	LV' ::= "(" VVopt ") LV''
	LV' ::= LV''

> parseLV' :: Posn -> XVp -> Vp -> Tokens -> (Ep, Tokens)
> parseLV' posn xv v t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseLV' posn xv v (LPAREN_TOKEN _:t1) =
>  let
>   (vv, t2) = parseVVopt     t1
>   (    t3) = parseRPAREN    t2
>   (e,  t4) = parseLV'' posn t3
>  in
>   (LETAp xv v vv e posn, t4)
> parseLV' posn [x] v t0 =
>  let
>   (e, t1) = parseLV'' posn t0
>  in
>   (LETp x v e posn, t1)
> parseLV' posn (x:xv) v t0 = (undefined, [ERR_TOKEN (exprPosn x) msg])
>  where msg = "cannot define multiple variables in a single binding"

	LV'' ::= "in" E
	LV'' ::= LV

> parseLV'' :: Posn -> Tokens -> (Ep, Tokens)
> parseLV'' posn t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseLV'' posn (IN_TOKEN _:t1) = parseE t1
> parseLV'' posn t0 = parseLV posn t0

	FV ::= epsilon			-- when followed by "in"
	FV ::= F FV

> parseFV :: Tokens -> (FVp, Tokens)
> parseFV t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseFV t0@(IN_TOKEN _:_) = ([], t0)
> parseFV t0 =
>  let
>   (f,  t1) = parseF  t0
>   (fv, t2) = parseFV t1
>  in
>   (f:fv, t2)

	VVopt ::= epsilon		-- when followed by ")"
	VVopt ::= VV

> parseVVopt :: Tokens -> (VVp, Tokens)
> parseVVopt t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseVVopt t0@(RPAREN_TOKEN _:_) = ([], t0)
> parseVVopt t0 = parseVV t0

	VV ::= V VV'

> parseVV :: Tokens -> (VVp, Tokens)
> parseVV t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseVV t0 =
>  let
>   (v,  t1) = parseV   t0
>   (vv, t2) = parseVV' t1
>  in
>   (v:vv, t2)

	VV' ::= "," VV
	VV' ::= epsilon

> parseVV' :: Tokens -> (VVp, Tokens)
> parseVV' (COMMA_TOKEN _:t1) = parseVV t1
> parseVV' t0 = ([], t0)

	XVopt ::= epsilon		-- when followed by ")"
	XVopt ::= XV

> parseXVopt :: Tokens -> (XVp, Tokens)
> parseXVopt t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseXVopt t0@(RPAREN_TOKEN _:_) = ([], t0)
> parseXVopt t0 = parseXV t0

	XV ::= X XV'

> parseXV :: Tokens -> (XVp, Tokens)
> parseXV t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseXV t0 =
>  let
>   (x,  t1) = parseX   t0
>   (xv, t2) = parseXV' t1
>  in
>   (x:xv, t2)

	XV' -> "," XV
	XV' -> epsilon

> parseXV' :: Tokens -> (XVp, Tokens)
> parseXV' (COMMA_TOKEN _:t1) = parseXV t1
> parseXV' t0 = ([], t0)

	F ::= X "(" XVopt ")" "=" E

> parseF :: Tokens -> (Fp, Tokens)
> parseF t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseF t0 =
>  let
>   (x,  t1) = parseX      t0
>   (    t2) = parseLPAREN t1
>   (xv, t3) = parseXVopt  t2
>   (    t4) = parseRPAREN t3
>   (    t5) = parseASSIGN t4
>   (e,  t6) = parseE      t5
>  in
>   (FUNp x xv e, t6)

	V -> <integer>
	V -> X

> parseV :: Tokens -> (Vp, Tokens)
> parseV t0@(ERR_TOKEN _ _:_) = (undefined, t0)
> parseV (INT_TOKEN posn x:t1) = (INTp x posn, t1)
> parseV t0 =
>  let
>   (x, t1) = parseX t0
>  in
>   (VARp x, t1)

	And some auxiliary productions for the terminals:

> parseX :: Tokens -> (Xp, Tokens)
> parseX (ID_TOKEN posn x:t1) = (IDp 0 x posn, t1)
> parseX t0 = (undefined, panic t0 "missing identifier")

> parseIN :: Tokens -> Tokens
> parseIN (IN_TOKEN _:t1) = t1
> parseIN t0 = panic t0 "missing keyword \"in\""

> parseTHEN :: Tokens -> Tokens
> parseTHEN (THEN_TOKEN _:t1) = t1
> parseTHEN t0 = panic t0 "missing keyword \"then\""

> parseELSE :: Tokens -> Tokens
> parseELSE (ELSE_TOKEN _:t1) = t1
> parseELSE t0 = panic t0 "missing keyword \"else\""

> parseLPAREN :: Tokens -> Tokens
> parseLPAREN (LPAREN_TOKEN _:t1) = t1
> parseLPAREN t0 = panic t0 "missing opening parenthese \"(\""

> parseRPAREN :: Tokens -> Tokens
> parseRPAREN (RPAREN_TOKEN _:t1) = t1
> parseRPAREN t0 = panic t0 "missing closing parenthese \")\""

> parseASSIGN :: Tokens -> Tokens
> parseASSIGN (ASSIGN_TOKEN _:t1) = t1
> parseASSIGN t0 = panic t0 "missing assignment operator \"=\""

  "panic" is a simple error handler that propagates errors embedded in the
  token stream up the parse tree:

> panic :: Tokens -> String -> Tokens
> panic t0@(ERR_TOKEN _ _:_) _ = t0
> panic (t:_) msg = [ERR_TOKEN (toknPosn t) msg]
