
  CHARACTER SET DEFINITIONS
  =========================

  Copyright  2000 Patryk Zadarnowski pat@jantar.org

  Redistribution  and  use  in  source  and  binary  forms,  with  or  without
  modification, are permitted provided  that the following conditions are met:
  (1) Redistributions of  source code must retain the  above copyright notice,
  this list  of conditions and the following  disclaimer.  (2) Redistributions
  in  binary form  must reproduce  the above  copyright notice,  this  list of
  conditions and  the following disclaimer  in the documentation  and/or other
  materials provided  with the distribution.  (3)  The name of  any author may
  not  be used  to  endorse or  promote  products derived  from this  software
  without their specific prior written permission.

  THIS SOFTWARE IS  PROVIDED ``AS IS AND ANY  EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
  AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
  SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
  INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
  CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
  ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

	$Id: Charset.lhs,v 1.1 2003/03/31 09:11:12 patrykz Exp $

   --------------------------------------------------------------------------


  OVERVIEW
  ========

  This module provides a collection of C-like character classification and
  mapping functions. Most of those are already implemented in the standard
  Char module; here they're just collected into a nice module, and augumented
  with additional functions I've found useful for parsing.

> module Charset (
>  isspace, isbreak, isblank,
>  isdigit, isbdigit, isodigit, isxdigit,
>  isalpha, islower, isupper, isalnum, isnumer,
>  isprint, isgraph, ispunct, iscntrl, isascii,
>  tolower, toupper, todigit, strtolower, strtoupper
> ) where

> import Common
> import Char (isSpace, isAlpha, isLower, isUpper, isAlphaNum,
>	       isPrint, isAscii, toUpper, toLower, digitToInt, ord, chr)

> isspace :: Char -> Bool
> isspace = isSpace

  Check if a character is a vertical space, using the Unicode recommended
  description of line break characters in programming langauges.  The
  two-character '\n\r' sequence has to be handled explicitely by each program.
  Hugs doesn't support characters > '\xff', so I had to trick the compiler
  by converting the type to an Int.

> isbreak :: Char -> Bool
> isbreak c = isbreak' (ord c)
>  where
>   isbreak' 0x000A = True
>   isbreak' 0x000B = True
>   isbreak' 0x000C = True
>   isbreak' 0x000D = True
>   isbreak' 0x0085 = True
>   isbreak' 0x2028 = True
>   isbreak' 0x2029 = True
>   isbreak' _ = False

  Check if a character is a horizonal space, ie. {space}\{break}:

> isblank :: Char -> Bool
> isblank c = isspace c /\ (not.isbreak) c

  Check if a character is an ASCII decimal digit:

> isdigit :: Char -> Bool
> isdigit '0' = True
> isdigit '1' = True
> isdigit '2' = True
> isdigit '3' = True
> isdigit '4' = True
> isdigit '5' = True
> isdigit '6' = True
> isdigit '7' = True
> isdigit '8' = True
> isdigit '9' = True
> isdigit  _  = False

  Check if a character is an ASCII binary digit:

> isbdigit :: Char -> Bool
> isbdigit '0' = True
> isbdigit '1' = True
> isbdigit  _  = False

  Check if a character is an ASCII octal digit:

> isodigit :: Char -> Bool
> isodigit '0' = True
> isodigit '1' = True
> isodigit '2' = True
> isodigit '3' = True
> isodigit '4' = True
> isodigit '5' = True
> isodigit '6' = True
> isodigit '7' = True
> isodigit  _  = False

  Check if a character is an ASCII hexadecimal digit:

> isxdigit :: Char -> Bool
> isxdigit '0' = True
> isxdigit '1' = True
> isxdigit '2' = True
> isxdigit '3' = True
> isxdigit '4' = True
> isxdigit '5' = True
> isxdigit '6' = True
> isxdigit '7' = True
> isxdigit '8' = True
> isxdigit '9' = True
> isxdigit 'a' = True
> isxdigit 'b' = True
> isxdigit 'c' = True
> isxdigit 'd' = True
> isxdigit 'e' = True
> isxdigit 'f' = True
> isxdigit 'A' = True
> isxdigit 'B' = True
> isxdigit 'C' = True
> isxdigit 'D' = True
> isxdigit 'E' = True
> isxdigit 'F' = True
> isxdigit  _  = False

  Check if a character is an Unicode letter:

> isalpha :: Char -> Bool
> isalpha = isAlpha

  Check if a character is a Unicode lower-case letter:

> islower :: Char -> Bool
> islower = isLower

  Check if a character is a Unicode upper-case letter:

> isupper :: Char -> Bool
> isupper = isUpper

  Check if a character is a Unicode alphanumeric character:

> isalnum :: Char -> Bool
> isalnum = isAlphaNum

  Check if a character is a Unicode numeric digit:

> isnumer :: Char -> Bool
> isnumer c = isalnum c /\ (not.isalpha) c

  Check if a character is a Unicode printable character:

> isprint :: Char -> Bool
> isprint = isPrint

  Check if a character is a graphical character:

> isgraph c = isprint c /\ (not.isspace) c

  Check if a character is a punctuator:

> ispunct :: Char -> Bool
> ispunct c | isalnum c = False
> ispunct c | otherwise = isgraph c

  Check if a character is a control character, ie. has no other special
  properties:

> iscntrl = not.isprint

  Check if a character is an ASCII character:

> isascii :: Char -> Bool
> isascii = isAscii

  Convert a character to its lower-case equivalent:

> tolower :: Char -> Char
> tolower = toLower

  Convert a character to its upper-case equivalent:

> toupper :: Char -> Char
> toupper = toUpper

  Convert a character to its numeric value:

> todigit :: Char -> Int
> todigit = digitToInt

  Convert a string to its lower-case equivalent:

> strtolower :: String -> String
> strtolower = map tolower

  Convert a string to its upper-case equivalent:

> strtoupper :: String -> String
> strtoupper = map toupper
