
  COMMON DEFINITIONS
  ==================

  Copyright  2000 Patryk Zadarnowski pat@jantar.org

  Redistribution  and  use  in  source  and  binary  forms,  with  or  without
  modification, are permitted provided  that the following conditions are met:
  (1) Redistributions of  source code must retain the  above copyright notice,
  this list  of conditions and the following  disclaimer.  (2) Redistributions
  in  binary form  must reproduce  the above  copyright notice,  this  list of
  conditions and  the following disclaimer  in the documentation  and/or other
  materials provided  with the distribution.  (3)  The name of  any author may
  not  be used  to  endorse or  promote  products derived  from this  software
  without their specific prior written permission.

  THIS SOFTWARE IS  PROVIDED ``AS IS AND ANY  EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
  AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
  SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
  INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
  CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
  ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

	$Id: Common.lhs,v 1.1 2003/03/31 09:11:12 patrykz Exp $

   --------------------------------------------------------------------------


  OVERVIEW
  ========

  This module defines some neat syntactic sugar missing from the Prelude.

> module Common (
>  Ints, Integers, Strings,
>  (/\), (\/), (<+>), (==>), (<=>), compl,
>  basename, dirname
> ) where

> infixr 3  /\
> infixr 2  \/, <+>, ==>, <=>


  LIST TYPES
  ==========

  I like to use plural names for lists:

> type Ints = [Int]
> type Integers = [Integer]
> type Strings = [String]


  BOOLEAN OPERATIONS
  ==================

  ... and Miranda ASCII art.  This defines lots of nice boolean operations I
  simply cannot live without. Or something. When it comes to style, only
  fools argue.

> class Bitstring a where
>  (/\)  :: a -> a -> a		-- bitwise conjunction (AND)
>  (\/)  :: a -> a -> a		-- bitwise disjunction (OR)
>  (<+>) :: a -> a -> a		-- bitwise sum (XOR)
>  (==>) :: a -> a -> a		-- bitwise modal conditional (IF)
>  (<=>) :: a -> a -> a		-- bitwise equivalence (IFF)
>  compl :: a -> a		-- bitwise complement (NOT)
>  
>  x <+> y = (x /\ compl y) \/ (compl x /\ y)
>  x ==> y = (compl x \/ y)
>  x <=> y = compl (x <+> y)

> instance Bitstring Bool where
>  (/\)  = (&&)
>  (\/)  = (||)
>  (<=>) = (==)
>  compl = not


  POSIX PATH FUNCTIONS
  ====================

  POSIX basename and dirname functions, which assume UNIX path conventions.

> basename :: String -> String
> basename string =
>  let
>   step3 = stripTrailingSlashes string
>   step4 = stripDirname step3
>  in
>   if allSlashes string then
>    "/"
>   else
>    step4

> dirname :: String -> String
> dirname string =
>  let
>   rstring = reverse string
>   step3 = stripLeadingSlashes rstring
>   (_, step5) = stripSegment step3
>   step7 = stripLeadingSlashes step5
>  in
>   if allSlashes string then
>    "/"
>   else if notElem '/' step3 then
>    "."
>   else if step7 == "" then
>    "/"
>   else
>    reverse step7

> allSlashes :: String -> Bool
> allSlashes "/" = True
> allSlashes ('/':rest) = allSlashes rest
> allSlashes _ = False

> stripLeadingSlashes :: String -> String
> stripLeadingSlashes ('/':rest) = stripLeadingSlashes rest
> stripLeadingSlashes string = string

> stripTrailingSlashes :: String -> String
> stripTrailingSlashes string = reverse (stripLeadingSlashes (reverse string))

> stripDirname :: String -> String
> stripDirname string = stripDirname' (True, string)

> stripDirname' :: (Bool, String) -> String
> stripDirname' (True, string) = stripDirname' (stripSegment string)
> stripDirname' (False, string) = string

> stripSegment :: String -> (Bool, String)
> stripSegment string = stripSegment' string string

> stripSegment' :: String -> String -> (Bool, String)
> stripSegment' _ ('/':rest) = (True, rest)
> stripSegment' orig (_:rest) = stripSegment' orig rest
> stripSegment' orig [] = (False, orig)
