
  A COMMAND-LINE ANF INTERPRETER
  ==============================

  Copyright  2000 Patryk Zadarnowski pat@jantar.org

  Redistribution  and  use  in  source  and  binary  forms,  with  or  without
  modification, are permitted provided  that the following conditions are met:
  (1) Redistributions of  source code must retain the  above copyright notice,
  this list  of conditions and the following  disclaimer.  (2) Redistributions
  in  binary form  must reproduce  the above  copyright notice,  this  list of
  conditions and  the following disclaimer  in the documentation  and/or other
  materials provided  with the distribution.  (3)  The name of  any author may
  not  be used  to  endorse or  promote  products derived  from this  software
  without their specific prior written permission.

  THIS SOFTWARE IS  PROVIDED ``AS IS AND ANY  EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
  AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
  SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
  INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
  CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
  ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

	$Id: Main.lhs,v 1.2 2003/03/31 09:09:39 patrykz Exp $

   --------------------------------------------------------------------------


> module Main (
>  main
> ) where

> import SSAtoANF
> import List (isSuffixOf)
> import System (getArgs, exitFailure)
> import Diag
> import qualified ANF
> import qualified SSA

> main :: IO ()
> main =
>  do
>   args <- getArgs
>   case args of
>    ["-a"] -> runANF (getContents) "stdin"
>    ["-s"] -> runSSA (getContents) "stdin"
>    ["-a",file] -> runANF (readFile file) file
>    ["-s",file] -> runSSA (readFile file) file
>    ["-t",file] -> translate (readFile file) file
>    [file] | isSuffixOf ".anf" file -> runANF (readFile file) file
>    [file] | isSuffixOf ".ssa" file -> runSSA (readFile file) file
>    _ -> printUsage

> runANF :: IO (String) -> String -> IO ()
> runANF reader name =
>  do
>   source <- reader
>   let
>    prog = ANF.rename (ANF.parse name source)
>   do
>    case ANF.check prog of
>     ANF.GOOD -> print (ANF.eval (ANF.strip prog))
>     ANF.BAD msgs -> emitDiagnostics msgs False source

> runSSA :: IO (String) -> String -> IO ()
> runSSA reader name =
>  do
>   source <- reader
>   let
>    prog = SSA.rename (SSA.parse name source)
>   do
>    case SSA.check prog of
>     SSA.GOOD -> print (SSA.eval (SSA.strip prog))
>     SSA.BAD msgs -> emitDiagnostics msgs False source

> translate :: IO (String) -> String -> IO ()
> translate reader name =
>  do
>   source <- reader
>   let
>    prog = SSA.rename (SSA.parse name source)
>   do
>    case SSA.check prog of
>     SSA.GOOD -> print (toANF (SSA.strip prog))
>     SSA.BAD msgs -> emitDiagnostics msgs False source

> printUsage :: IO ()
> printUsage =
>  do
>   putStr "usage: lab -a [<ANF source file>]\n\
>          \       lab -s [<SSA source file>]\n\
>	   \       lab -t [<SSA source file>]\n\
>          \       lab <source file>\n\
>	   \\n\
>          \When using \"lab\" without any flags, source files must have\n\
>	   \the extension \".anf\" or \".ssa\".\n"
>   exitFailure
