
  Copyright  2000 Patryk Zadarnowski pat@jantar.org

  Redistribution  and  use  in  source  and  binary  forms,  with  or  without
  modification, are permitted provided  that the following conditions are met:
  (1) Redistributions of  source code must retain the  above copyright notice,
  this list  of conditions and the following  disclaimer.  (2) Redistributions
  in  binary form  must reproduce  the above  copyright notice,  this  list of
  conditions and  the following disclaimer  in the documentation  and/or other
  materials provided  with the distribution.  (3)  The name of  any author may
  not  be used  to  endorse or  promote  products derived  from this  software
  without their specific prior written permission.

  THIS SOFTWARE IS  PROVIDED ``AS IS AND ANY  EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
  AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
  SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
  INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
  CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
  ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

  $Id: SCC.lhs,v 1.2 2002/12/12 13:15:27 patrykz Exp $

   --------------------------------------------------------------------------


  SPARSE CONDITIONAL CONSTANT PROPAGATION
  =======================================

> module SCC (
>  scc
> ) where

> import FiniteMap
> import ANF

> data Va = TOP | BOTTOM | ICa Integer | CLa XV E X | PRIMa ([Va] -> Va) X

> instance Eq Va where
>  (==) TOP TOP = True
>  (==) BOTTOM BOTTOM = True
>  (==) (ICa x) (ICa y) = (x == y)
>  (==) (CLa _ _ x) (CLa _ _ y) = (x == y)
>  (==) (PRIMa _ x) (PRIMa _ y) = (x == y)

> instance Show Va where
>  showsPrec prec TOP = \s -> "<top>" ++ s
>  showsPrec prec BOTTOM = \s -> "<bottom>" ++ s
>  showsPrec prec (ICa x) = showsPrec prec x
>  showsPrec prec (CLa _ _ x) = \s -> "<function " ++ x ++ ">" ++ s
>  showsPrec prec (PRIMa _ x) = \s -> "<primitive " ++ x ++ ">" ++ s

> meet :: Va -> Va -> Va

> meet TOP (ICa x) = TOP
> meet TOP (CLa _ _ x) = TOP
> meet TOP (PRIMa _ x) = TOP

> meet (ICa x) TOP = TOP
> meet (CLa _ _ x) TOP = TOP
> meet (PRIMa _ x) TOP = TOP

> meet va va' = if va == va' then va else BOTTOM

> type Gamma = FiniteMap X Va
> type Rho = FiniteMap X Va
> type Omega = [F]

> glookup :: Gamma -> V -> Va
> glookup gamma (VAR x) = lookupWithDefaultFM gamma TOP x
> glookup gamma (IC c) = ICa c

> rlookup :: Rho -> X -> Va
> rlookup rho f = lookupWithDefaultFM rho TOP f

> vars :: [X] -> [V]
> vars = map (\ x -> VAR x)

> type Occ = (X -> Omega)

> analyze :: Occ -> E -> Gamma -> Rho -> Omega -> (E, Gamma, Rho, Omega)

> analyze occ (RET v []) gamma rho omega = (glookup gamma v, gamma, rho, omega)

> analyze occ (RET v vv) gamma rho omega =
>  let
>   va = glookup gamma v
>   vva = map (glookup gamma) vv
>  in
>   case va of
>    TOP -> (TOP, gamma, rho, omega) -- is that right?
>    CLa vx e f | length vx == length vv ->
>     let
>      vva' = map (glookup gamma) (vars vx)
>      va' = rlookup rho f
>     in
>      if zipWith meet vva vva' == vva' then
>	(z, gamma, rho, omega)
>      else
>	(z, addListToFM_C meet gamma (zip vx vva), rho, (FUN f vx e:omega)
>    PRIMa op f -> (op vva, gamma, rho, omega)
>    _ -> (BOTTOM, gamma, rho, omega)

> analyze occ (LET x v [] e) gamma rho omega =
>  let
>   va = glookup gamma v
>   va' = glookup gamma (VAR x)
>  in
>   if meet va va' == va' then
>    analyze occ e gamma rho omega
>   else
>    analyze occ e (addToFM_C meet gamma x va) rho (occ x ++ omega)

> analyze occ (LET x v vv e) gamma rho omega =
>  let
>   (va, gamma', rho', omega') = analyze occ (RET v vv) gamma rho omega
>   va' = glookup gamma' (VAR x)
>  in
>   if meet va va' == va' then
>    analyze occ e gamma' rho' omega'
>   else
>    analyze occ e (addToFM_C meet gamma' x va) rho' (occ x ++ omega')

> analyze occ (LETREC vf e) gamma rho omega =
>  let
>   gamma' = addListToFM gamma (map (\ (FUN x vx e) -> (x, CLa vx e x)) vf)
>   (_, gamma'', rho'', omega'') = analyze e gamma' rho []
>   (gamma''', rho''') = processOmega omega'' gamma'' rho''
>  in
>   analyze e gamma''' rho''' omega
>  where
>   processOmega [] gamma rho = (gamma, rho)
>   processOmega (FUN f vx e:omega) gamma rho =
>    let
>     (va, gamma', rho', omega') = analyze e gamma rho omega
>     va' = rlookup rho' va
>    in
>     if va == va' then -- should we use "meet"?
>      processOmega (occ f ++ omega') gamma' rho'
>     else
>      processOmega omega' gamma' (addToFM rho' f va) -- should we use "meet"?

> analyze (IF v e1 e2) gamma rho omega =
>  let
>   va = glookup gamma v
>  in
>   case va of
>    TOP -> (TOP, gamma, rho, omega)
>    ICa 0 -> analyze e2 gamma rho omega
>    ICa _ -> analyze e1 gamma rho omega
>    _ ->
>     let
>      (va', gamma', rho', omega') = analyze e1 gamma rho omega
>      (va'', gamma'', rho'', omega'') = analyze e2 gamma' rho' omega'
>     in
>      (meet va' va'', gamma'', rho'', omega'')

> occ :: X -> [FUN]
> occ x =
>  --TODO

TODO: convert omega into a set to avoid duplicates