
  SSA TO ANF CONVERSION ALGORITHM
  ===============================

  Copyright  2000 Patryk Zadarnowski pat@jantar.org

  Redistribution  and  use  in  source  and  binary  forms,  with  or  without
  modification, are permitted provided  that the following conditions are met:
  (1) Redistributions of  source code must retain the  above copyright notice,
  this list  of conditions and the following  disclaimer.  (2) Redistributions
  in  binary form  must reproduce  the above  copyright notice,  this  list of
  conditions and  the following disclaimer  in the documentation  and/or other
  materials provided  with the distribution.  (3)  The name of  any author may
  not  be used  to  endorse or  promote  products derived  from this  software
  without their specific prior written permission.

  THIS SOFTWARE IS  PROVIDED ``AS IS AND ANY  EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
  AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
  SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
  INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
  CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
  ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

	$Id: SSAtoANF.lhs,v 1.2 2003/03/31 09:09:39 patrykz Exp $

   --------------------------------------------------------------------------


  OVERVIEW
  ========

  This module implements the SSA to ANF transformation.

> module SSAtoANF (
>  toANF
> ) where

> import FiniteMap
> import qualified SSA
> import qualified ANF

  The label environment:

> type Lambda = FiniteMap Int SSA.E

> bextend :: Lambda -> SSA.X -> SSA.E -> Lambda
> bextend lambda (SSA.ID id) e = addToFM lambda id e

> blookup :: Lambda -> SSA.X -> SSA.E
> blookup lambda x@(SSA.ID id) = lookupWithDefaultFM lambda err id
>  where err = error ("undefined label <" ++ show x ++ ">")

  The translation function collects SSA procedures into an all-encompassing
  outermost "letrec":

> toANF :: SSA.P -> ANF.E
> toANF (SSA.PROG fv e) =
>  ANF.LETREC (toANFfv fv) (toANFe emptyFM SSA.START e)

  Convert an SSA procedure to an ANF function:

> toANFf :: SSA.F -> ANF.F
> toANFf (SSA.PROC x xv e bv) =
>  ANF.FUN (toANFx x) (toANFxv xv) (toANFp emptyFM SSA.START e bv) 

> toANFfv :: SSA.FV -> ANF.FV
> toANFfv fv = map (toANFf) fv

  Convert an SSA procedure or group to an inner "letrec":

> toANFp :: Lambda -> SSA.L -> SSA.E -> SSA.BV -> ANF.E
> toANFp lambda l e bv = ANF.LETREC (toANFbv lambda' bv) (toANFe lambda' l e)
>  where lambda' = beta emptyFM bv

  Convert an SSA group to an ANF function:

> toANFb :: Lambda -> SSA.B -> ANF.F
> toANFb lambda (SSA.BLOCK x e) =
>  ANF.FUN (toANFx x) (toANFa e []) (toANFe lambda (SSA.LABEL x) e)
> toANFb lambda (SSA.GROUP x e bv) =
>  ANF.FUN (toANFx x) (toANFa e []) (toANFp lambda (SSA.LABEL x) e bv)

> toANFbv :: Lambda -> SSA.BV -> ANF.FV
> toANFbv lambda bv = map (toANFb lambda) bv

  Convert an SSA expression to an ANF expression:

> toANFe :: Lambda -> SSA.L -> SSA.E -> ANF.E
> toANFe lambda l (SSA.PHI x gv e) = toANFe lambda l e
> toANFe lambda l (SSA.LET x v e) =
>  ANF.LET (toANFx x) (toANFv v) (toANFe lambda l e)
> toANFe lambda l (SSA.LETA xv v vv e) =
>  ANF.LETA (toANFxv xv) (toANFv v) (toANFvv vv) (toANFe lambda l e)
> toANFe lambda l (SSA.RET vv) = ANF.RET (toANFvv vv)
> toANFe lambda l (SSA.RETA v vv) = ANF.RETA (toANFv v) (toANFvv vv)
> toANFe lambda l (SSA.GOTO x) =
>  ANF.RETA (ANF.VAR (toANFx x)) (toANFg l (blookup lambda x) [])
> toANFe lambda l (SSA.IF v e1 e2) =
>  ANF.IF (toANFv v) (toANFe lambda l e1) (toANFe lambda l e2)

  Construct a list of function parameters from "phi"-nodes:

> toANFa :: SSA.E -> ANF.XV -> ANF.XV
> toANFa (SSA.PHI x gv e) xv = toANFx x : toANFa e xv
> toANFa (SSA.LET x v e) xv = toANFa e xv
> toANFa (SSA.LETA x v vv e) xv = toANFa e xv
> toANFa (SSA.IF v e1 e2) xv = toANFa e1 (toANFa e2 xv)
> toANFa e xv = xv

  Construct an argument list for a translated jump:

> toANFg :: SSA.L -> SSA.E -> ANF.VV -> ANF.VV
> toANFg l (SSA.PHI x gv e) vv = toANFv (kappa gv l) : toANFg l e vv
> toANFg l (SSA.LET x v e) vv = toANFg l e vv
> toANFg l (SSA.LETA xv v _ e) vv = toANFg l e vv
> toANFg l (SSA.IF v e1 e2) vv = toANFg l e1 (toANFg l e2 vv)
> toANFg l e vv = vv

  Convert an SSA identifier:

> toANFx :: SSA.X -> ANF.X
> toANFx (SSA.ID x) = ANF.ID x

> toANFxv :: SSA.XV -> ANF.XV
> toANFxv xv = map (toANFx) xv

  Convert an SSA variable:

> toANFv :: SSA.V -> ANF.V
> toANFv (SSA.VAR x) = ANF.VAR (toANFx x)
> toANFv (SSA.INT x) = ANF.INT x

> toANFvv :: SSA.VV -> ANF.VV
> toANFvv vv = map (toANFv) vv

  Search the list of "phi" parameters for a label:

> kappa :: SSA.GV -> SSA.L -> SSA.V
> kappa (SSA.FROM l1 v:gv) l2 = if l1 == l2 then v else kappa gv l2

  Look up the label environment:

> beta :: Lambda -> SSA.BV -> Lambda
> beta lambda bv = foldl (beta') lambda bv
>  where
>   beta' lambda b = bextend lambda (SSA.blockLabel b) (SSA.blockExpr b)
